@@ -1,5 +1,9 @@
 #define _GNU_SOURCE
 
+#include "system.h"
+
+#include <rpm/rpmlog.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
@@ -10,6 +14,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sys/mman.h>
+#include <errno.h>
 
 #include <endian.h>
 
@@ -165,7 +170,6 @@ static int rpmidxReadHeader(rpmidxdb idxdb);
 
 static int rpmidxMap(rpmidxdb idxdb)
 {
-    struct stat stb;
     if (idxdb->xdb) {
 	if (rpmxdbMapBlob(idxdb->xdb, idxdb->xdbid, idxdb->rdonly ? O_RDONLY : O_RDWR, mapcb, idxdb))
 	    return RPMRC_FAIL;
@@ -174,6 +178,8 @@ static int rpmidxMap(rpmidxdb idxdb)
 	    return RPMRC_FAIL;
 	}
     } else {
+#ifdef IDXDB_FILESUPPORT
+	struct stat stb;
 	size_t size;
 	void *mapped;
 	if (fstat(idxdb->fd, &stb))
@@ -187,6 +193,9 @@ static int rpmidxMap(rpmidxdb idxdb)
 	if (mapped == MAP_FAILED)
 	    return RPMRC_FAIL;
 	set_mapped(idxdb, mapped, (unsigned int)stb.st_size);
+#else
+	return RPMRC_FAIL;
+#endif
     }
     return RPMRC_OK;
 }
@@ -198,14 +207,19 @@ static void rpmidxUnmap(rpmidxdb idxdb)
     if (idxdb->xdb) {
 	rpmxdbUnmapBlob(idxdb->xdb, idxdb->xdbid);
     } else {
+#ifdef IDXDB_FILESUPPORT
 	size_t size = idxdb->file_size;
 	/* round up for munmap */
 	size = (size + idxdb->pagesize - 1) & ~(idxdb->pagesize - 1);
 	munmap(idxdb->head_mapped, size);
 	set_mapped(idxdb, 0, 0);
+#else
+	return;
+#endif
     }
 }
 
+#ifdef IDXDB_FILESUPPORT
 static int rpmidxReadHeader(rpmidxdb idxdb);
 
 /* re-open file to get the new version */
@@ -230,9 +244,12 @@ static int rpmidxHandleObsolete(rpmidxdb idxdb)
     idxdb->fd = nfd;
     return rpmidxReadHeader(idxdb);	/* re-try with new file */
 }
+#endif
 
 static int rpmidxReadHeader(rpmidxdb idxdb)
 {
+    unsigned int version;
+
     if (idxdb->head_mapped) {
 	if (le2ha(idxdb->head_mapped + IDXDB_OFFSET_GENERATION) == idxdb->generation)
 	    return RPMRC_OK;
@@ -246,8 +263,17 @@ static int rpmidxReadHeader(rpmidxdb idxdb)
 	rpmidxUnmap(idxdb);
 	return RPMRC_FAIL;
     }
+    version = le2ha(idxdb->head_mapped + IDXDB_OFFSET_VERSION);
+    if (version != IDXDB_VERSION) {
+	rpmlog(RPMLOG_ERR, _("rpmidx: Version mismatch. Expected version: %u. "
+	    "Found version: %u\n"), IDXDB_VERSION, version);
+	rpmidxUnmap(idxdb);
+	return RPMRC_FAIL;
+    }
+#ifdef IDXDB_FILESUPPORT
     if (!idxdb->xdb && le2ha(idxdb->head_mapped + IDXDB_OFFSET_OBSOLETE))
 	return rpmidxHandleObsolete(idxdb);
+#endif
     idxdb->generation = le2ha(idxdb->head_mapped + IDXDB_OFFSET_GENERATION);
     idxdb->nslots     = le2ha(idxdb->head_mapped + IDXDB_OFFSET_NSLOTS);
     idxdb->usedslots  = le2ha(idxdb->head_mapped + IDXDB_OFFSET_USEDSLOTS);
@@ -310,6 +336,7 @@ static inline void bumpGeneration(rpmidxdb idxdb)
     h2lea(idxdb->generation, idxdb->head_mapped + IDXDB_OFFSET_GENERATION);
 }
 
+#ifdef IDXDB_FILESUPPORT
 static int createempty(rpmidxdb idxdb, off_t off, size_t size)
 {
     char buf[4096];
@@ -324,6 +351,7 @@ static int createempty(rpmidxdb idxdb, off_t off, size_t size)
 	return RPMRC_FAIL;
     return RPMRC_OK;
 }
+#endif
 
 /*** Key management ***/
 
@@ -428,6 +456,7 @@ static int addkeypage(rpmidxdb idxdb) {
 	if (rpmxdbResizeBlob(idxdb->xdb, idxdb->xdbid, idxdb->file_size + addsize))
 	    return RPMRC_FAIL;
     } else {
+#ifdef IDXDB_FILESUPPORT
 	/* don't use ftruncate because we want to create a "backed" page */
 	void *newaddr;
 	size_t oldsize, newsize;
@@ -442,6 +471,9 @@ static int addkeypage(rpmidxdb idxdb) {
 	if (newaddr == MAP_FAILED)
 	    return RPMRC_FAIL;
 	set_mapped(idxdb, newaddr, idxdb->file_size + addsize);
+#else
+	return RPMRC_FAIL;
+#endif
     }
     return RPMRC_OK;
 }
@@ -608,6 +640,7 @@ static int rpmidxRebuildInternal(rpmidxdb idxdb)
 	    return RPMRC_FAIL;
 	}
     } else {
+#ifdef IDXDB_FILESUPPORT
 	void *mapped;
 	nidxdb->filename = malloc(strlen(idxdb->filename) + 8);
 	if (!nidxdb->filename)
@@ -632,6 +665,9 @@ static int rpmidxRebuildInternal(rpmidxdb idxdb)
 	    return RPMRC_FAIL;
 	}
 	set_mapped(nidxdb, mapped, file_size);
+#else
+	return RPMRC_FAIL;
+#endif
     }
 
     /* copy all entries */
@@ -677,7 +713,9 @@ static int rpmidxRebuildInternal(rpmidxdb idxdb)
 	if (nidxdb->xdb) {
 	    rpmxdbResizeBlob(nidxdb->xdb, nidxdb->xdbid, xfile_size);
 	} else {
-	    ftruncate(nidxdb->fd, xfile_size);
+	    if (ftruncate(nidxdb->fd, xfile_size)) {
+		rpmlog(RPMLOG_WARNING, _("truncate failed: %s\n"), strerror(errno));
+	    }
 	}
     }
 
@@ -688,6 +726,7 @@ static int rpmidxRebuildInternal(rpmidxdb idxdb)
 	    return RPMRC_FAIL;
 	idxdb->xdbid = nidxdb->xdbid;
     } else {
+#ifdef IDXDB_FILESUPPORT
 	if (rename(nidxdb->filename, idxdb->filename)) {
 	    close(nidxdb->fd);
 	    unlink(nidxdb->filename);
@@ -702,6 +741,9 @@ static int rpmidxRebuildInternal(rpmidxdb idxdb)
 	free(nidxdb->filename);
 	close(idxdb->fd);
 	idxdb->fd = nidxdb->fd;
+#else
+	return RPMRC_FAIL;
+#endif
     }
     if (rpmidxReadHeader(idxdb))
 	return RPMRC_FAIL;
@@ -993,11 +1035,15 @@ static int rpmidxInitInternal(rpmidxdb idxdb)
 	if (rc && rc != RPMRC_NOTFOUND)
 	    return rc;
     } else {
+#ifdef IDXDB_FILESUPPORT
 	struct stat stb; 
 	if (stat(idxdb->filename, &stb))
 	    return RPMRC_FAIL;
 	if (stb.st_size)	/* somebody else was faster */
 	    return rpmidxHandleObsolete(idxdb);
+#else
+	return RPMRC_FAIL;
+#endif
     }
     return rpmidxRebuildInternal(idxdb);
 }
@@ -1043,6 +1089,7 @@ static int rpmidxInit(rpmidxdb idxdb)
 
 int rpmidxOpen(rpmidxdb *idxdbp, rpmpkgdb pkgdb, const char *filename, int flags, int mode)
 {
+#ifdef IDXDB_FILESUPPORT
     struct stat stb;
     rpmidxdb idxdb;
 
@@ -1058,6 +1105,8 @@ int rpmidxOpen(rpmidxdb *idxdbp, rpmpkgdb pkgdb, const char *filename, int flags
     if ((flags & (O_RDONLY|O_RDWR)) == O_RDONLY)
 	idxdb->rdonly = 1;
     if ((idxdb->fd = open(filename, flags, mode)) == -1) {
+	free(idxdb->filename);
+	free(idxdb);
 	return RPMRC_FAIL;
     }   
     if (fstat(idxdb->fd, &stb)) {
@@ -1080,6 +1129,9 @@ int rpmidxOpen(rpmidxdb *idxdbp, rpmpkgdb pkgdb, const char *filename, int flags
     }
     *idxdbp = idxdb;
     return RPMRC_OK;
+#else
+    return RPMRC_FAIL;
+#endif
 }
 
 int rpmidxOpenXdb(rpmidxdb *idxdbp, rpmpkgdb pkgdb, rpmxdb xdb, unsigned int xdbtag)
