@@ -1,5 +1,9 @@
 #define _GNU_SOURCE
 
+#include "system.h"
+
+#include <rpm/rpmlog.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
@@ -11,6 +15,7 @@
 #include <stdlib.h>
 #include <sys/mman.h>
 #include <endian.h>
+#include <libgen.h>
 
 #include "rpmxdb.h"
 
@@ -115,7 +120,7 @@ static int mapslot(rpmxdb xdb, struct xdb_slot *slot)
 	shift = off & (xdb->systempagesize - 1);
 	off -= shift;
 	size += shift;
-	size = (size + xdb->systempagesize - 1) & (xdb->systempagesize - 1);
+	size = (size + xdb->systempagesize - 1) & ~(xdb->systempagesize - 1);
     }
     mapped = mmap(0, size, slot->mapflags, MAP_SHARED, xdb->fd, off);
     if (mapped == MAP_FAILED)
@@ -136,7 +141,7 @@ static void unmapslot(rpmxdb xdb, struct xdb_slot *slot)
 	size_t shift = off & (xdb->systempagesize - 1);
 	mapped -= shift;
 	size += shift;
-	size = (size + xdb->systempagesize - 1) & (xdb->systempagesize - 1);
+	size = (size + xdb->systempagesize - 1) & ~(xdb->systempagesize - 1);
     }
     munmap(mapped, size);
     slot->mapped = 0;
@@ -155,9 +160,9 @@ static int remapslot(rpmxdb xdb, struct xdb_slot *slot, unsigned int newpagecnt)
 	shift = off & (xdb->systempagesize - 1);
 	off -= shift;
 	oldsize += shift;
-	oldsize = (oldsize + xdb->systempagesize - 1) & (xdb->systempagesize - 1);
+	oldsize = (oldsize + xdb->systempagesize - 1) & ~(xdb->systempagesize - 1);
 	newsize += shift;
-	newsize = (newsize + xdb->systempagesize - 1) & (xdb->systempagesize - 1);
+	newsize = (newsize + xdb->systempagesize - 1) & ~(xdb->systempagesize - 1);
     }
     if (slot->mapped)
 	mapped = mremap(slot->mapped - shift, oldsize, newsize, MREMAP_MAYMOVE);
@@ -185,7 +190,7 @@ static int rpmxdbReadHeader(rpmxdb xdb)
 {
     struct xdb_slot *slot;
     unsigned int header[XDB_HEADER_SIZE / sizeof(unsigned int)];
-    unsigned int slotnpages, pagesize, generation, usergeneration;
+    unsigned int slotnpages, pagesize, generation, usergeneration, version;
     unsigned int page, *lastfreep;
     unsigned char *pageptr;
     struct xdb_slot *slots, **usedslots, *lastslot;
@@ -209,6 +214,13 @@ static int rpmxdbReadHeader(rpmxdb xdb)
     }
     if (le2ha((unsigned char *)header + XDB_OFFSET_MAGIC) != XDB_MAGIC)
 	return RPMRC_FAIL;
+    version = le2ha((unsigned char *)header + XDB_OFFSET_VERSION);
+    if (version != XDB_VERSION) {
+	rpmlog(RPMLOG_ERR, _("rpmxdb: Version mismatch. Expected version: %u. "
+	    "Found version: %u\n"), XDB_VERSION, version);
+	return RPMRC_FAIL;
+    }
+
     generation = le2ha((unsigned char *)header + XDB_OFFSET_GENERATION);
     slotnpages = le2ha((unsigned char *)header + XDB_OFFSET_SLOTNPAGES);
     pagesize = le2ha((unsigned char *)header + XDB_OFFSET_PAGESIZE);
@@ -349,6 +361,7 @@ static int rpmxdbWriteHeader(rpmxdb xdb)
     h2lea(xdb->generation, xdb->mapped + XDB_OFFSET_GENERATION);
     h2lea(xdb->slotnpages, xdb->mapped + XDB_OFFSET_SLOTNPAGES);
     h2lea(xdb->pagesize, xdb->mapped + XDB_OFFSET_PAGESIZE);
+    h2lea(xdb->usergeneration, xdb->mapped + XDB_OFFSET_USERGENERATION);
     return RPMRC_OK;
 }
 
@@ -501,6 +514,36 @@ int rpmxdbOpen(rpmxdb *xdbp, rpmpkgdb pkgdb, const char *filename, int flags, in
 	free(xdb);
 	return RPMRC_FAIL;
     }
+    if (flags & O_CREAT) {
+	char *filenameCopy;
+	DIR *pdir;
+
+	if ((filenameCopy = strdup(xdb->filename)) == NULL) {
+	    close(xdb->fd);
+	    free(xdb->filename);
+	    free(xdb);
+	    return RPMRC_FAIL;
+	}
+
+	if ((pdir = opendir(dirname(filenameCopy))) == NULL) {
+	    free(filenameCopy);
+	    close(xdb->fd);
+	    free(xdb->filename);
+	    free(xdb);
+	    return RPMRC_FAIL;
+	}
+
+	if (fsync(dirfd(pdir)) == -1) {
+	    closedir(pdir);
+	    free(filenameCopy);
+	    close(xdb->fd);
+	    free(xdb->filename);
+	    free(xdb);
+	    return RPMRC_FAIL;
+	}
+	closedir(pdir);
+	free(filenameCopy);
+    }
     if (fstat(xdb->fd, &stb)) {
 	close(xdb->fd);
 	free(xdb->filename);
@@ -1118,7 +1161,7 @@ int rpmxdbSetUserGeneration(rpmxdb xdb, unsigned int usergeneration)
     if (rpmxdbLockReadHeader(xdb, 1))
         return RPMRC_FAIL;
     /* sync before the update */
-    if (xdb->dofsync && fdatasync(xdb->fd)) {
+    if (xdb->dofsync && fsync(xdb->fd)) {
 	rpmxdbUnlock(xdb, 1);
 	return RPMRC_FAIL;
     }
