@@ -207,47 +207,9 @@ static int fsmSetFCaps(const char *path, const char *captxt)
     return rc;
 }
 
-/* Check dest is the same, empty and regular file with writeonly permissions */
-static int linkSane(FD_t wfd, const char *dest)
+static void wfd_close(FD_t *wfdp)
 {
-    struct stat sb, lsb;
-
-    return (fstat(Fileno(wfd), &sb) == 0 && sb.st_size == 0 &&
-	    (sb.st_mode & ~S_IFMT) == S_IWUSR &&
-	    lstat(dest, &lsb) == 0 && S_ISREG(lsb.st_mode) &&
-	    sb.st_dev == lsb.st_dev && sb.st_ino == lsb.st_ino);
-}
-
-/** \ingroup payload
- * Create file from payload stream.
- * @return		0 on success
- */
-static int expandRegular(rpmfi fi, const char *dest, rpmpsm psm, int exclusive, int nodigest, int nocontent)
-{
-    FD_t wfd = NULL;
-    int rc = 0;
-
-    /* Create the file with 0200 permissions (write by owner). */
-    {
-	mode_t old_umask = umask(0577);
-	wfd = Fopen(dest, exclusive ? "wx.ufdio" : "a.ufdio");
-	umask(old_umask);
-
-	/* If reopening, make sure the file is what we expect */
-	if (!exclusive && wfd != NULL && !linkSane(wfd, dest)) {
-	    rc = RPMERR_OPEN_FAILED;
-	    goto exit;
-	}
-    }
-    if (Ferror(wfd)) {
-	rc = RPMERR_OPEN_FAILED;
-	goto exit;
-    }
-
-    if (!nocontent)
-	rc = rpmfiArchiveReadToFilePsm(fi, wfd, nodigest, psm);
-exit:
-    if (wfd) {
+    if (wfdp && *wfdp) {
 	int myerrno = errno;
 	static int oneshot = 0;
 	static int flush_io = 0;
@@ -256,18 +218,58 @@ exit:
 	    oneshot = 1;
 	}
 	if (flush_io) {
-	    int fdno = Fileno(wfd);
+	    int fdno = Fileno(*wfdp);
 	    fsync(fdno);
 	}
-	Fclose(wfd);
+	Fclose(*wfdp);
+	*wfdp = NULL;
 	errno = myerrno;
     }
+}
+
+static int wfd_open(FD_t *wfdp, const char *dest)
+{
+    int rc = 0;
+    /* Create the file with 0200 permissions (write by owner). */
+    {
+	mode_t old_umask = umask(0577);
+	*wfdp = Fopen(dest, "wx.ufdio");
+	umask(old_umask);
+    }
+    if (Ferror(*wfdp)) {
+	rc = RPMERR_OPEN_FAILED;
+	goto exit;
+    }
+
+    return 0;
+
+exit:
+    wfd_close(wfdp);
+    return rc;
+}
+
+/** \ingroup payload
+ * Create file from payload stream.
+ * @return		0 on success
+ */
+static int expandRegular(rpmfi fi, const char *dest, rpmpsm psm, int nodigest)
+{
+    FD_t wfd = NULL;
+    int rc;
+
+    rc = wfd_open(&wfd, dest);
+    if (rc != 0)
+        goto exit;
+
+    rc = rpmfiArchiveReadToFilePsm(fi, wfd, nodigest, psm);
+    wfd_close(&wfd);
+exit:
     return rc;
 }
 
 static int fsmMkfile(rpmfi fi, const char *dest, rpmfiles files,
 		     rpmpsm psm, int nodigest, int *setmeta,
-		     int * firsthardlink)
+		     int * firsthardlink, FD_t *firstlinkfile)
 {
     int rc = 0;
     int numHardlinks = rpmfiFNlink(fi);
@@ -276,7 +278,7 @@ static int fsmMkfile(rpmfi fi, const char *dest, rpmfiles files,
 	/* Create first hardlinked file empty */
 	if (*firsthardlink < 0) {
 	    *firsthardlink = rpmfiFX(fi);
-	    rc = expandRegular(fi, dest, psm, 1, nodigest, 1);
+	    rc = wfd_open(firstlinkfile, dest);
 	} else {
 	    /* Create hard links for others */
 	    char *fn = rpmfilesFN(files, *firsthardlink);
@@ -291,10 +293,11 @@ static int fsmMkfile(rpmfi fi, const char *dest, rpmfiles files,
        existing) file with content */
     if (numHardlinks<=1) {
 	if (!rc)
-	    rc = expandRegular(fi, dest, psm, 1, nodigest, 0);
+	    rc = expandRegular(fi, dest, psm, nodigest);
     } else if (rpmfiArchiveHasContent(fi)) {
 	if (!rc)
-	    rc = expandRegular(fi, dest, psm, 0, nodigest, 0);
+	    rc = rpmfiArchiveReadToFilePsm(fi, *firstlinkfile, nodigest, psm);
+	wfd_close(firstlinkfile);
 	*firsthardlink = -1;
     } else {
 	*setmeta = 0;
@@ -861,6 +864,7 @@ int rpmPackageFilesInstall(rpmts ts, rpmte te, rpmfiles files,
     int nodigest = (rpmtsFlags(ts) & RPMTRANS_FLAG_NOFILEDIGEST) ? 1 : 0;
     int nofcaps = (rpmtsFlags(ts) & RPMTRANS_FLAG_NOCAPS) ? 1 : 0;
     int firsthardlink = -1;
+    FD_t firstlinkfile = NULL;
     int skip;
     rpmFileAction action;
     char *tid = NULL;
@@ -932,7 +936,7 @@ int rpmPackageFilesInstall(rpmts ts, rpmte te, rpmfiles files,
             if (S_ISREG(sb.st_mode)) {
 		if (rc == RPMERR_ENOENT) {
 		    rc = fsmMkfile(fi, fpath, files, psm, nodigest,
-				   &setmeta, &firsthardlink);
+				   &setmeta, &firsthardlink, &firstlinkfile);
 		}
             } else if (S_ISDIR(sb.st_mode)) {
                 if (rc == RPMERR_ENOENT) {
@@ -970,7 +974,8 @@ int rpmPackageFilesInstall(rpmts ts, rpmte te, rpmfiles files,
 	    /* we skip the hard linked file containing the content */
 	    /* write the content to the first used instead */
 	    char *fn = rpmfilesFN(files, firsthardlink);
-	    rc = expandRegular(fi, fn, psm, 0, nodigest, 0);
+	    rc = rpmfiArchiveReadToFilePsm(fi, firstlinkfile, nodigest, psm);
+	    wfd_close(&firstlinkfile);
 	    firsthardlink = -1;
 	    free(fn);
 	}
