@@ -249,7 +249,7 @@ static rpmRC runInstScript(rpmpsm psm, rpmTagVal scriptTag)
 
     if (script) {
 	headerGet(h, RPMTAG_INSTPREFIXES, &pfx, HEADERGET_ALLOC|HEADERGET_ARGV);
-	rc = runScript(psm->ts, psm->te, pfx.data, script, psm->scriptArg, -1);
+	rc = runScript(psm->ts, psm->te, h, pfx.data, script, psm->scriptArg, -1);
 	rpmtdFreeData(&pfx);
     }
 
@@ -312,8 +312,7 @@ static rpmRC handleOneTrigger(rpmts ts, rpmte te, rpmsenseFlags sense,
 		rpmScript script = rpmScriptFromTriggerTag(trigH,
 			     triggertag(sense), RPMSCRIPT_NORMALTRIGGER, tix);
 		arg1 += countCorrection;
-		rc = runScript(ts, te, pfx.data, script, arg1, arg2);
-
+		rc = runScript(ts, te, trigH, pfx.data, script, arg1, arg2);
 		if (triggersAlreadyRun != NULL)
 		    triggersAlreadyRun[tix] = 1;
 
@@ -360,8 +359,8 @@ static rpmRC runTriggers(rpmpsm psm, rpmsenseFlags sense)
 	rpmdbMatchIterator mi;
 
 	mi = rpmtsInitIterator(ts, RPMDBI_TRIGGERNAME, N, 0);
-	while((triggeredH = rpmdbNextIterator(mi)) != NULL) {
-	    nerrors += handleOneTrigger(ts, psm->te, sense, h, triggeredH,
+	while ((triggeredH = rpmdbNextIterator(mi)) != NULL) {
+	    nerrors += handleOneTrigger(ts, NULL, sense, h, triggeredH,
 					0, numPackage, NULL);
 	}
 	rpmdbFreeIterator(mi);
@@ -403,7 +402,7 @@ static rpmRC runImmedTriggers(rpmpsm psm, rpmsenseFlags sense)
 	
 	    mi = rpmtsInitIterator(ts, RPMDBI_NAME, trigName, 0);
 
-	    while((sourceH = rpmdbNextIterator(mi)) != NULL) {
+	    while ((sourceH = rpmdbNextIterator(mi)) != NULL) {
 		nerrors += handleOneTrigger(psm->ts, psm->te,
 				sense, sourceH, h,
 				psm->countCorrection,
@@ -479,8 +478,10 @@ static rpmpsm rpmpsmNew(rpmts ts, rpmte te, pkgGoal goal)
     if (psm->total == 0)
 	psm->total = 100;
 
-    rpmlog(RPMLOG_DEBUG, "%s: %s has %d files\n", pkgGoalString(goal),
+    if (goal == PKG_INSTALL || goal == PKG_ERASE) {
+	rpmlog(RPMLOG_DEBUG, "%s: %s has %d files\n", pkgGoalString(goal),
 	    rpmteNEVRA(psm->te), rpmfilesFC(psm->files));
+    }
 
     return psm;
 }
@@ -814,7 +815,7 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
 
 static const char * pkgGoalString(pkgGoal goal)
 {
-    switch(goal) {
+    switch (goal) {
     case PKG_INSTALL:	return "  install";
     case PKG_ERASE:	return "    erase";
     case PKG_VERIFY:	return "   verify";
