@@ -33,6 +33,7 @@
 #include "lib/fprint.h"
 #include "lib/header_internal.h"	/* XXX for headerSetInstance() */
 #include "lib/backend/dbiset.h"
+#include "lib/misc.h"
 #include "debug.h"
 
 #undef HASHTYPE
@@ -310,75 +311,20 @@ static rpmdb rpmdbRock;
 static rpmdbMatchIterator rpmmiRock;
 static rpmdbIndexIterator rpmiiRock;
 
-int rpmdbCheckTerminate(int terminate)
+void rpmAtExit(void)
 {
-    sigset_t newMask, oldMask;
-    static int terminating = 0;
+    rpmdb db;
+    rpmdbMatchIterator mi;
+    rpmdbIndexIterator ii;
 
-    if (terminating) return 0;
+    while ((mi = rpmmiRock) != NULL)
+	rpmdbFreeIterator(mi);
 
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
+    while ((ii = rpmiiRock) != NULL)
+	rpmdbIndexIteratorFree(ii);
 
-    if (rpmsqIsCaught(SIGINT) > 0
-     || rpmsqIsCaught(SIGQUIT) > 0
-     || rpmsqIsCaught(SIGHUP) > 0
-     || rpmsqIsCaught(SIGTERM) > 0
-     || rpmsqIsCaught(SIGPIPE) > 0
-     || terminate)
-	terminating = 1;
-
-    if (terminating) {
-	rpmdb db;
-	rpmdbMatchIterator mi;
-	rpmdbIndexIterator ii;
-
-	while ((mi = rpmmiRock) != NULL)
-	    rpmdbFreeIterator(mi);
-
-	while ((ii = rpmiiRock) != NULL)
-	    rpmdbIndexIteratorFree(ii);
-
-	while ((db = rpmdbRock) != NULL)
-	    (void) rpmdbClose(db);
-    }
-    sigprocmask(SIG_SETMASK, &oldMask, NULL);
-    return terminating;
-}
-
-int rpmdbCheckSignals(void)
-{
-    if (rpmdbCheckTerminate(0)) {
-	rpmlog(RPMLOG_DEBUG, "Exiting on signal...\n");
-	exit(EXIT_FAILURE);
-    }
-    return 0;
-}
-
-/**
- * Block all signals, returning previous signal mask.
- */
-static int blockSignals(sigset_t * oldMask)
-{
-    sigset_t newMask;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, oldMask);
-    (void) sigdelset(&newMask, SIGINT);
-    (void) sigdelset(&newMask, SIGQUIT);
-    (void) sigdelset(&newMask, SIGHUP);
-    (void) sigdelset(&newMask, SIGTERM);
-    (void) sigdelset(&newMask, SIGPIPE);
-    return sigprocmask(SIG_BLOCK, &newMask, NULL);
-}
-
-/**
- * Restore signal mask.
- */
-static int unblockSignals(sigset_t * oldMask)
-{
-    (void) rpmdbCheckSignals();
-    return sigprocmask(SIG_SETMASK, oldMask, NULL);
+    while ((db = rpmdbRock) != NULL)
+	(void) rpmdbClose(db);
 }
 
 rpmop rpmdbOp(rpmdb rpmdb, rpmdbOpX opx)
@@ -479,11 +425,7 @@ int rpmdbClose(rpmdb db)
     db = _free(db);
 
     if (rpmdbRock == NULL) {
-	(void) rpmsqEnable(-SIGHUP, NULL);
-	(void) rpmsqEnable(-SIGINT, NULL);
-	(void) rpmsqEnable(-SIGTERM, NULL);
-	(void) rpmsqEnable(-SIGQUIT, NULL);
-	(void) rpmsqEnable(-SIGPIPE, NULL);
+	rpmsqActivate(0);
     }
 exit:
     return rc;
@@ -564,11 +506,7 @@ static int openDatabase(const char * prefix,
     rc = rpmioMkpath(rpmdbHome(db), 0755, getuid(), getgid());
     if (rc == 0) {
 	if (rpmdbRock == NULL) {
-	    (void) rpmsqEnable(SIGHUP, NULL);
-	    (void) rpmsqEnable(SIGINT, NULL);
-	    (void) rpmsqEnable(SIGTERM, NULL);
-	    (void) rpmsqEnable(SIGQUIT, NULL);
-	    (void) rpmsqEnable(SIGPIPE, NULL);
+	    rpmsqActivate(1);
 	}
 
 	/* Just the primary Packages database opened here */
@@ -734,11 +672,8 @@ static rpmRC rpmdbFindByFile(rpmdb db, dbiIndex dbi, const char *filespec,
 	    if (!skip) {
 		const char *dirName = dirNames[dirIndexes[num]];
 		if (fpLookupEquals(fpc, fp1, dirName, baseNames[num])) {
-		    struct dbiIndexItem_s rec = { 
-			.hdrNum = dbiIndexRecordOffset(allMatches, i),
-			.tagNum = dbiIndexRecordFileNumber(allMatches, i),
-		    };
-		    dbiIndexSetAppend(*matches, &rec, 1, 0);
+		    dbiIndexSetAppendOne(*matches, dbiIndexRecordOffset(allMatches, i),
+					 dbiIndexRecordFileNumber(allMatches, i), 0);
 		}
 	    }
 
@@ -1040,15 +975,13 @@ static int miFreeHeader(rpmdbMatchIterator mi, dbiIndex dbi)
 	}
 
 	if (hdrBlob != NULL && rpmrc != RPMRC_FAIL) {
-	    sigset_t signalMask;
-
-	    blockSignals(&signalMask);
+	    rpmsqBlock(SIG_BLOCK);
 	    dbCtrl(mi->mi_db, DB_CTRL_LOCK_RW);
 	    rc = pkgdbPut(dbi, mi->mi_dbc, mi->mi_prevoffset,
 			  hdrBlob, hdrLen);
 	    dbCtrl(mi->mi_db, DB_CTRL_INDEXSYNC);
 	    dbCtrl(mi->mi_db, DB_CTRL_UNLOCK_RW);
-	    unblockSignals(&signalMask);
+	    rpmsqBlock(SIG_UNBLOCK);
 
 	    if (rc) {
 		rpmlog(RPMLOG_ERR,
@@ -1105,7 +1038,7 @@ rpmdbMatchIterator rpmdbFreeIterator(rpmdbMatchIterator mi)
 
     mi = _free(mi);
 
-    (void) rpmdbCheckSignals();
+    (void) rpmsqPoll();
 
     return NULL;
 }
@@ -1712,10 +1645,8 @@ int rpmdbAppendIterator(rpmdbMatchIterator mi,
     if (mi->mi_set == NULL)
 	mi->mi_set = dbiIndexSetNew(nHdrNums);
 
-    for (unsigned int i = 0; i < nHdrNums; i++) {
-	struct dbiIndexItem_s rec = { .hdrNum = hdrNums[i], .tagNum = 0 };
-	dbiIndexSetAppend(mi->mi_set, &rec, 1, 0);
-    }
+    for (unsigned int i = 0; i < nHdrNums; i++)
+	dbiIndexSetAppendOne(mi->mi_set, hdrNums[i], 0, 0);
     return 0;
 }
 
@@ -1831,7 +1762,7 @@ rpmdbMatchIterator rpmdbInitIterator(rpmdb db, rpmDbiTagVal rpmtag,
     rpmdbMatchIterator mi = NULL;
 
     if (db != NULL) {
-	(void) rpmdbCheckSignals();
+	(void) rpmsqPoll();
 
 	if (rpmtag == RPMDBI_PACKAGES)
 	    mi = pkgdbIterInit(db, keyp, keylen);
@@ -1854,7 +1785,7 @@ rpmdbMatchIterator rpmdbInitPrefixIterator(rpmdb db, rpmDbiTagVal rpmtag,
 	return NULL;
 
     if (db != NULL && rpmtag != RPMDBI_PACKAGES) {
-	(void) rpmdbCheckSignals();
+	(void) rpmsqPoll();
 
 
 	if (indexOpen(db, dbtag, 0, &dbi) == 0) {
@@ -1940,7 +1871,7 @@ rpmdbIndexIterator rpmdbIndexIteratorInit(rpmdb db, rpmDbiTag rpmtag)
     if (db == NULL)
 	return NULL;
 
-    (void) rpmdbCheckSignals();
+    (void) rpmsqPoll();
 
     if (indexOpen(db, rpmtag, 0, &dbi))
 	return NULL;
@@ -2119,7 +2050,6 @@ int rpmdbRemove(rpmdb db, unsigned int hdrNum)
     dbiIndex dbi = NULL;
     dbiCursor dbc = NULL;
     Header h;
-    sigset_t signalMask;
     int ret = 0;
 
     if (db == NULL)
@@ -2140,7 +2070,7 @@ int rpmdbRemove(rpmdb db, unsigned int hdrNum)
     if (pkgdbOpen(db, 0, &dbi))
 	return 1;
 
-    (void) blockSignals(&signalMask);
+    rpmsqBlock(SIG_BLOCK);
     dbCtrl(db, DB_CTRL_LOCK_RW);
 
     /* Remove header from primary index */
@@ -2162,7 +2092,7 @@ int rpmdbRemove(rpmdb db, unsigned int hdrNum)
 
     dbCtrl(db, DB_CTRL_INDEXSYNC);
     dbCtrl(db, DB_CTRL_UNLOCK_RW);
-    (void) unblockSignals(&signalMask);
+    rpmsqBlock(SIG_UNBLOCK);
 
     headerFree(h);
 
@@ -2199,7 +2129,7 @@ static rpmRC updateRichDepCB(void *cbdata, rpmrichParseType type,
 	data->nargv++;
 	_free(name);
     }
-    if (type == RPMRICH_PARSE_OP && op == RPMRICHOP_IF) {
+    if (type == RPMRICH_PARSE_OP && (op == RPMRICHOP_IF || op == RPMRICHOP_UNLESS)) {
 	/* save nargv in case of ELSE */
 	data->nargv_level[data->level - 1] = data->nargv;
 	data->neg ^= 1;
@@ -2216,7 +2146,7 @@ static rpmRC updateRichDepCB(void *cbdata, rpmrichParseType type,
 	}
 	data->neg ^= 1;
     }
-    if (type == RPMRICH_PARSE_LEAVE && op == RPMRICHOP_IF) {
+    if (type == RPMRICH_PARSE_LEAVE && (op == RPMRICHOP_IF || op == RPMRICHOP_UNLESS)) {
 	data->neg ^= 1;
     }
     return RPMRC_OK;
@@ -2370,7 +2300,6 @@ static rpmRC indexPut(dbiIndex dbi, rpmTagVal rpmtag, unsigned int hdrNum, Heade
 
 int rpmdbAdd(rpmdb db, Header h)
 {
-    sigset_t signalMask;
     dbiIndex dbi = NULL;
     dbiCursor dbc = NULL;
     unsigned int hdrNum = 0;
@@ -2391,7 +2320,7 @@ int rpmdbAdd(rpmdb db, Header h)
     if (ret)
 	goto exit;
 	
-    (void) blockSignals(&signalMask);
+    rpmsqBlock(SIG_BLOCK);
     dbCtrl(db, DB_CTRL_LOCK_RW);
 
     /* Add header to primary index */
@@ -2415,7 +2344,7 @@ int rpmdbAdd(rpmdb db, Header h)
 
     dbCtrl(db, DB_CTRL_INDEXSYNC);
     dbCtrl(db, DB_CTRL_UNLOCK_RW);
-    (void) unblockSignals(&signalMask);
+    rpmsqBlock(SIG_UNBLOCK);
 
     /* If everything ok, mark header as installed now */
     if (ret == 0) {
@@ -2432,18 +2361,10 @@ exit:
     return ret;
 }
 
-/*
- * Remove DB4 environment (and lock), ie the equivalent of 
- * rm -f <prefix>/<dbpath>/__db.???
- * Environment files not existing is not an error, failure to unlink is,
- * return zero on success.
- * TODO/FIX: push this down to db3.c where it belongs
- */
-static int cleanDbenv(const char *prefix, const char *dbpath)
+static int rpmdbRemoveFiles(char * pattern)
 {
+    int rc = 0;
     ARGV_t paths = NULL, p;
-    int rc = 0; 
-    char *pattern = rpmGetPath(prefix, "/", dbpath, "/__db.???", NULL);
 
     if (rpmGlob(pattern, NULL, &paths) == 0) {
 	for (p = paths; *p; p++) {
@@ -2451,99 +2372,114 @@ static int cleanDbenv(const char *prefix, const char *dbpath)
 	}
 	argvFree(paths);
     }
+    return rc;
+}
+
+static int rpmdbRemoveDatabase(const char *dbpath)
+{
+    int rc = 0; 
+    char *pattern;
+
+    pattern = rpmGetPath(dbpath, "/*", NULL);
+    rc += rpmdbRemoveFiles(pattern);
     free(pattern);
+    pattern = rpmGetPath(dbpath, "/.??*", NULL);
+    rc += rpmdbRemoveFiles(pattern);
+    free(pattern);
+    
+    rc += rmdir(dbpath);
     return rc;
 }
 
-static int unlinkTag(const char * prefix, const char *dbpath, rpmTagVal dbtag)
+static int rpmdbMoveDatabase(const char * prefix, const char * srcdbpath,
+			     const char * dbpath, const char * tmppath)
 {
-    int rc = 0;
-    const char * base = rpmTagGetName(dbtag);
-    char * path = rpmGetPath(prefix, "/", dbpath, "/", base, NULL);
-    if (access(path, F_OK) == 0)
-	rc = unlink(path);
-    free(path);
-    return rc;
-}
+    int rc = -1;
+    int xx;
+    char *src = rpmGetPath(prefix, "/", srcdbpath, NULL);
+    char *old = rpmGetPath(prefix, "/", tmppath, NULL);
+    char *dest = rpmGetPath(prefix, "/", dbpath, NULL);
 
-static int rpmdbRemoveDatabase(const char * prefix, const char * dbpath)
-{ 
-    char *path;
-    int xx = 0;
-    /* create a handle but dont actually open */
-    rpmdb db = newRpmdb(prefix, dbpath, O_RDONLY, 0644, RPMDB_FLAG_REBUILD);
+    char * oldkeys = rpmGetPath(old, "/", "pubkeys", NULL);
+    char * destkeys = rpmGetPath(dest, "/", "pubkeys", NULL);
 
-    xx = unlinkTag(prefix, dbpath, RPMDBI_PACKAGES);
-    for (int i = 0; i < db->db_ndbi; i++) {
-	xx += unlinkTag(prefix, dbpath, db->db_tags[i]);
+    xx = rename(dest, old);
+    if (xx) {
+	goto exit;
+    }
+    xx = rename(src, dest);
+    if (xx) {
+	rpmlog(RPMLOG_ERR, _("could not move new database in place\n"));
+	xx = rename(old, dest);
+	if (xx) {
+	    rpmlog(RPMLOG_ERR, _("could also not restore old database from %s\n"),
+		   old);
+	    rpmlog(RPMLOG_ERR, _("replace files in %s with files from %s "
+				 "to recover\n"), dest, old);
+	}
+	goto exit;
     }
-    cleanDbenv(prefix, dbpath);
 
-    path = rpmGetPath(prefix, "/", dbpath, NULL);
-    xx += rmdir(path);
-    free(path);
-    rpmdbClose(db);
-
-    return (xx != 0);
-}
-
-static int renameTag(const char * prefix,
-		     const char * olddbpath, const char *newdbpath,
-		     const char * base)
-{
-    int xx, rc = 0;
-    char *src = rpmGetPath(prefix, "/", olddbpath, "/", base, NULL);
-    char *dest = rpmGetPath(prefix, "/", newdbpath, "/", base, NULL);
-    struct stat st;
-
-    if (access(src, F_OK) == 0) {
-	/*
-	 * Restore uid/gid/mode/security context if possible.
-	 */
-	if (stat(dest, &st) < 0)
-	    if (stat(src, &st) < 0)
-		goto exit;
-
-	if ((xx = rename(src, dest)) != 0) {
-	    rc = 1;
+    if (access(oldkeys, F_OK ) != -1) {
+	xx = rename(oldkeys, destkeys);
+	if (xx) {
+	    rpmlog(RPMLOG_ERR, _("Could not get public keys from %s\n"), oldkeys);
 	    goto exit;
 	}
-	xx = chown(dest, st.st_uid, st.st_gid);
-	xx = chmod(dest, (st.st_mode & 07777));
-
-	/* XXX: we should call file prepare plugins here for selinux etc! */
     }
 
-exit:
-    free(src);
-    free(dest);
+    xx = rpmdbRemoveDatabase(old);
+    if (xx) {
+	rpmlog(RPMLOG_ERR, _("could not delete old database at %s\n"), old);
+    }
+
+    rc = 0;
+
+ exit:
+    _free(src);
+    _free(old);
+    _free(dest);
+    _free(oldkeys);
+    _free(destkeys);
     return rc;
 }
 
-static int rpmdbMoveDatabase(const char * prefix,
-			     const char * olddbpath, const char * newdbpath)
+static int rpmdbSetPermissions(char * src, char * dest)
 {
-    int rc = 0;
-    sigset_t sigMask;
-    /* create a handle but dont actually open */
-    rpmdb db = newRpmdb(prefix, newdbpath, O_RDONLY, 0644, RPMDB_FLAG_REBUILD);
+    struct dirent *dp;
+    DIR *dfd;
 
-    blockSignals(&sigMask);
-    rc = renameTag(prefix, olddbpath, newdbpath, rpmTagGetName(RPMDBI_PACKAGES));
-    for (int i = 0; i < db->db_ndbi; i++) {
-	rc += renameTag(prefix, olddbpath, newdbpath, rpmTagGetName(db->db_tags[i]));
+    struct stat st;
+    int xx, rc = -1;
+    char * filepath;
+    
+    if (stat(dest, &st) < 0)
+	    goto exit;
+    if (stat(src, &st) < 0)
+	    goto exit;
+
+    if ((dfd = opendir(dest)) == NULL) {
+	goto exit;
     }
-#ifdef ENABLE_NDB
-    rc += renameTag(prefix, olddbpath, newdbpath, "Packages.db");
-    rc += renameTag(prefix, olddbpath, newdbpath, "Index.db");
-#endif
 
-    cleanDbenv(prefix, olddbpath);
-    cleanDbenv(prefix, newdbpath);
-    rpmdbClose(db);
-
-    unblockSignals(&sigMask);
+    rc = 0;
+    while ((dp = readdir(dfd)) != NULL) {
+	if (!strcmp(dp->d_name, "..")) {
+	    continue;
+	}
+	filepath = rpmGetPath(dest, "/", dp->d_name, NULL);
+	xx = chown(filepath, st.st_uid, st.st_gid);
+	rc += xx;
+	if (!strcmp(dp->d_name, ".")) {
+	    xx = chmod(filepath, (st.st_mode & 07777));
+	} else {
+	    xx = chmod(filepath, (st.st_mode & 07666));
+	}
+	rc += xx;
+	_free(filepath);
+    }
 
+ exit:
     return rc;
 }
 
@@ -2553,12 +2489,12 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     rpmdb olddb;
     char * dbpath = NULL;
     char * rootdbpath = NULL;
+    char * tmppath = NULL;
     rpmdb newdb;
     char * newdbpath = NULL;
     char * newrootdbpath = NULL;
     int nocleanup = 1;
     int failed = 0;
-    int removedir = 0;
     int rc = 0;
 
     dbpath = rpmGetPath("%{?_dbpath}", NULL);
@@ -2586,7 +2522,6 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	rc = 1;
 	goto exit;
     }
-    removedir = 1;
 
     if (openDatabase(prefix, dbpath, &olddb,
 		     O_RDONLY, 0644, RPMDB_FLAG_REBUILD)) {
@@ -2598,6 +2533,10 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	rc = 1;
 	goto exit;
     }
+    if (rpmdbOpenAll(newdb)) {
+	rc = 1;
+	goto exit;
+    }
 
     {	Header h = NULL;
 	rpmdbMatchIterator mi;
@@ -2621,10 +2560,12 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	    }
 
 	    /* Deleted entries are eliminated in legacy headers by copy. */
-	    {	Header nh = (headerIsEntry(h, RPMTAG_HEADERIMAGE)
-				? headerCopy(h) : NULL);
-		rc = rpmdbAdd(newdb, (nh ? nh : h));
+	    if (headerIsEntry(h, RPMTAG_HEADERIMAGE)) {
+		Header nh = headerReload(headerCopy(h), RPMTAG_HEADERIMAGE);
+		rc = rpmdbAdd(newdb, h);
 		headerFree(nh);
+	    } else {
+		rc = rpmdbAdd(newdb, h);
 	    }
 
 	    if (rc) {
@@ -2647,15 +2588,20 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 		_("failed to rebuild database: original database "
 		"remains in place\n"));
 
-	rpmdbRemoveDatabase(prefix, newdbpath);
+	rpmdbRemoveDatabase(newrootdbpath);
 	rc = 1;
 	goto exit;
-    } else if (!nocleanup) {
-	if (rpmdbMoveDatabase(prefix, newdbpath, dbpath)) {
+    } else {
+	rpmdbSetPermissions(dbpath, newdbpath);
+    }
+
+    if (!nocleanup) {
+	rasprintf(&tmppath, "%sold.%d", dbpath, (int) getpid());
+	if (rpmdbMoveDatabase(prefix, newdbpath, dbpath, tmppath)) {
 	    rpmlog(RPMLOG_ERR, _("failed to replace old database with new "
 			"database!\n"));
 	    rpmlog(RPMLOG_ERR, _("replace files in %s with files from %s "
-			"to recover"), dbpath, newdbpath);
+			"to recover\n"), dbpath, newdbpath);
 	    rc = 1;
 	    goto exit;
 	}
@@ -2663,13 +2609,9 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     rc = 0;
 
 exit:
-    if (removedir && !(rc == 0 && nocleanup)) {
-	if (rmdir(newrootdbpath))
-	    rpmlog(RPMLOG_ERR, _("failed to remove directory %s: %s\n"),
-			newrootdbpath, strerror(errno));
-    }
     free(newdbpath);
     free(dbpath);
+    free(tmppath);
     free(newrootdbpath);
     free(rootdbpath);
 
