@@ -60,6 +60,7 @@ enum rpmfileAttrs_e {
     RPMFILE_README	= (1 <<  8),	/*!< from %%readme */
     /* bits 9-10 unused */
     RPMFILE_PUBKEY	= (1 << 11),	/*!< from %%pubkey */
+    RPMFILE_ARTIFACT	= (1 << 12),	/*!< from %%artifact */
 };
 
 typedef rpmFlags rpmfileAttrs;
@@ -138,14 +139,17 @@ typedef rpmFlags rpmfiFlags;
     (RPMFI_NOFILECLASS | RPMFI_NOFILEDEPS | RPMFI_NOFILELANGS | \
      RPMFI_NOFILECOLORS | RPMFI_NOFILEVERIFYFLAGS)
 
-#define RPMFI_FLAGS_ONLY_FILENAMES \
+#define RPMFI_FLAGS_FILETRIGGER \
     (RPMFI_NOFILECLASS | RPMFI_NOFILEDEPS | RPMFI_NOFILELANGS | \
      RPMFI_NOFILEUSER | RPMFI_NOFILEGROUP | RPMFI_NOFILEMODES | \
      RPMFI_NOFILESIZES | RPMFI_NOFILECAPS | RPMFI_NOFILELINKTOS | \
      RPMFI_NOFILEDIGESTS | RPMFI_NOFILEMTIMES | RPMFI_NOFILERDEVS | \
-     RPMFI_NOFILEINODES | RPMFI_NOFILESTATES | RPMFI_NOFILECOLORS | \
+     RPMFI_NOFILEINODES | RPMFI_NOFILECOLORS | \
      RPMFI_NOFILEVERIFYFLAGS | RPMFI_NOFILEFLAGS)
 
+#define RPMFI_FLAGS_ONLY_FILENAMES \
+    (RPMFI_FLAGS_FILETRIGGER | RPMFI_NOFILESTATES)
+
 typedef enum rpmFileIter_e {
     RPMFI_ITER_FWD	= 0,
     RPMFI_ITER_BACK	= 1,
