@@ -4,10 +4,6 @@
 #include <sys/wait.h>
 #include <errno.h>
 #include <unistd.h>
-#include <sys/resource.h>
-#if defined(__linux__)
-#include <sys/syscall.h>        /* For ionice */
-#endif
 
 #include <rpm/rpmfileutil.h>
 #include <rpm/rpmmacro.h>
@@ -166,6 +162,7 @@ static void doScriptExec(ARGV_const_t argv, ARGV_const_t prefixes,
     int xx;
     int open_max;
 
+    /* SIGPIPE is ignored in rpm, reset to default for the scriptlet */
     (void) signal(SIGPIPE, SIG_DFL);
 
     /* XXX Force FD_CLOEXEC on all inherited fdno's. */
@@ -271,7 +268,6 @@ static rpmRC runExtScript(rpmPlugins plugins, ARGV_const_t prefixes,
     const char *line;
     char *mline = NULL;
     rpmRC rc = RPMRC_FAIL;
-    struct sigaction newact, oldact;
 
     rpmlog(RPMLOG_DEBUG, "%s: scriptlet start\n", sname);
 
@@ -319,12 +315,6 @@ static rpmRC runExtScript(rpmPlugins plugins, ARGV_const_t prefixes,
 	goto exit;
     }
 
-    /* Ignore SIGPIPE during execution of scriptlets */
-    sigemptyset(&newact.sa_mask);
-    newact.sa_flags = 0;
-    newact.sa_handler = SIG_IGN;
-    sigaction(SIGPIPE, &newact, &oldact);
-
     pid = fork();
     if (pid == (pid_t) -1) {
 	rpmlog(RPMLOG_ERR, _("Couldn't fork %s: %s\n"),
@@ -337,32 +327,6 @@ static rpmRC runExtScript(rpmPlugins plugins, ARGV_const_t prefixes,
 	fclose(in);
 	dup2(inpipe[0], STDIN_FILENO);
 
-        /* If RPM was invoked with nice and/or ionice, the scripts that we run
-         * will be also nice'd/ionice'd.  This is terrible if you restart any
-         * daemon (e.g. mysqld), so let's reset this to default values before
-         * taking any actions.
-         */
-
-        /* Call for resetting nice priority. */
-        int ret;
-        ret = setpriority(PRIO_PROCESS, 0, 0);
-        if (ret == -1) {
-            rpmlog(RPMLOG_WARNING, _("Unable to reset nice value: %s"),
-                strerror(errno));
-        }
-
-        /* Call for resetting IO priority. */
-        #if defined(__linux__)
-        /* Defined at include/linux/ioprio.h */
-        const int _IOPRIO_WHO_PROCESS = 1;
-        const int _IOPRIO_CLASS_NONE = 0;
-        ret = syscall(SYS_ioprio_set, _IOPRIO_WHO_PROCESS, 0, _IOPRIO_CLASS_NONE);
-        if (ret == -1) {
-            rpmlog(RPMLOG_WARNING, _("Unable to reset I/O priority: %s"),
-                strerror(errno));
-        }
-        #endif
-
 	/* Run scriptlet post fork hook for all plugins */
 	if (rpmpluginsCallScriptletForkPost(plugins, *argvp[0], RPMSCRIPTLET_FORK | RPMSCRIPTLET_EXEC) != RPMRC_FAIL) {
 	    doScriptExec(*argvp, prefixes, scriptFd, out);
@@ -436,9 +400,6 @@ exit:
     }
     free(mline);
 
-    /* Restore SIGPIPE handler */
-    sigaction(SIGPIPE, &oldact, NULL);
-
     return rc;
 }
 
@@ -602,7 +563,7 @@ rpmScript rpmScriptFromTriggerTag(Header h, rpmTagVal triggerTag,
     struct rpmtd_s tscripts, tprogs, tflags;
     headerGetFlags hgflags = HEADERGET_MINMEM;
 
-    switch(tm) {
+    switch (tm) {
 	case RPMSCRIPT_NORMALTRIGGER:
 	    headerGet(h, RPMTAG_TRIGGERSCRIPTS, &tscripts, hgflags);
 	    headerGet(h, RPMTAG_TRIGGERSCRIPTPROG, &tprogs, hgflags);
