@@ -19,6 +19,7 @@
 #include <rpm/rpmds.h>
 #include <rpm/rpmfi.h>
 #include <rpm/rpmlog.h>
+#include <rpm/rpmsq.h>
 #include <rpm/rpmte.h>
 
 #include "rpmio/digest.h"
@@ -415,12 +416,12 @@ static int makePubkeyHeader(rpmts ts, rpmPubkey key, rpmPubkey *subkeys,
 
     /* Build header elements. */
     v = pgpHexStr(pubp->signid, sizeof(pubp->signid)); 
-    r = pgpHexStr(pubp->time, sizeof(pubp->time));
     userid = pubp->userid ? pubp->userid : "none";
-    keytime = pgpGrab(pubp->time, sizeof(pubp->time));
+    keytime = pubp->time;
 
     rasprintf(&n, "gpg(%s)", v+8);
     rasprintf(&u, "gpg(%s)", userid);
+    rasprintf(&r, "%x", keytime);
     rasprintf(&evr, "%d:%s-%s", pubp->version, v, r);
 
     headerPutString(h, RPMTAG_PUBKEYS, enc);
@@ -458,9 +459,9 @@ static int makePubkeyHeader(rpmts ts, rpmPubkey key, rpmPubkey *subkeys,
 
 	pgpkey = rpmPubkeyPgpDigParams(subkeys[i]);
 	v = pgpHexStr(pgpkey->signid, sizeof(pgpkey->signid));
-	r = pgpHexStr(pgpkey->time, sizeof(pgpkey->time));
 
 	rasprintf(&n, "gpg(%s)", v+8);
+	rasprintf(&r, "%x", pgpkey->time);
 	rasprintf(&evr, "%d:%s-%s", pubp->version, v, r);
 
 	headerPutString(h, RPMTAG_PROVIDENAME, n);
@@ -1055,6 +1056,8 @@ rpmts rpmtsCreate(void)
 
     ts->trigs2run = rpmtriggersCreate(10);
 
+    ts->min_writes = rpmExpandNumeric("%{_minimize_writes}");
+
     return rpmtsLink(ts);
 }
 
@@ -1143,6 +1146,8 @@ rpmtxn rpmtxnBegin(rpmts ts, rpmtxnFlags flags)
 	txn->lock = ts->lock;
 	txn->flags = flags;
 	txn->ts = rpmtsLink(ts);
+	if (txn->flags & RPMTXN_WRITE)
+	    rpmsqBlock(SIG_BLOCK);
     }
     
     return txn;
@@ -1152,6 +1157,8 @@ rpmtxn rpmtxnEnd(rpmtxn txn)
 {
     if (txn) {
 	rpmlockRelease(txn->lock);
+	if (txn->flags & RPMTXN_WRITE)
+	    rpmsqBlock(SIG_UNBLOCK);
 	rpmtsFree(txn->ts);
 	free(txn);
     }
