@@ -84,6 +84,8 @@ struct rpmts_s {
     int nrefs;			/*!< Reference count. */
 
     rpmtriggers trigs2run;   /*!< Transaction file triggers */
+
+    int min_writes;             /*!< macro minimize_writes used */
 };
 
 #ifdef __cplusplus
@@ -117,7 +119,7 @@ RPM_GNUC_INTERNAL
 rpmRC rpmtsSetupTransactionPlugins(rpmts ts);
 
 RPM_GNUC_INTERNAL
-rpmRC runScript(rpmts ts, rpmte te, ARGV_const_t prefixes,
+rpmRC runScript(rpmts ts, rpmte te, Header h, ARGV_const_t prefixes,
 		       rpmScript script, int arg1, int arg2);
 
 #ifdef __cplusplus
