@@ -1,4 +1,4 @@
-#!/usr/bin/python
+from __future__ import with_statement
 
 import sys
 import rpm
@@ -53,9 +53,8 @@ class TransactionSet(TransactionSetCore):
 
     def _f2hdr(self, item):
         if isinstance(item, _string_types):
-            f = open(item)
-            header = self.hdrFromFdno(f)
-            f.close()
+            with open(item) as f:
+                header = self.hdrFromFdno(f)
         elif isinstance(item, rpm.hdr):
             header = item
         else:
@@ -139,11 +138,11 @@ class TransactionSet(TransactionSetCore):
             needflags = rpm.RPMSENSE_ANY
             if len(needs) == 3:
                 needop = needs[1]
-                if needop.find('<') >= 0:
+                if '<' in needop:
                     needflags |= rpm.RPMSENSE_LESS
-                if needop.find('=') >= 0:
+                if '=' in needop:
                     needflags |= rpm.RPMSENSE_EQUAL
-                if needop.find('>') >= 0:
+                if '>' in needop:
                     needflags |= rpm.RPMSENSE_GREATER
                 needver = needs[2]
             else:
