@@ -1,5 +1,4 @@
 #include "system.h"
-const char *__progname;
 
 #include <errno.h>
 #include <libgen.h>
@@ -430,7 +429,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     /* Override default BUILD value for _builddir */
     if (buildInPlace) {
 	char *cwd = rpmGetCwd();
-	addMacro(NULL, "_builddir", NULL, cwd, 0);
+	rpmPushMacro(NULL, "_builddir", NULL, cwd, 0);
 	free(cwd);
     }
 
@@ -460,7 +459,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	    dir = xstrdup(arg);
 	}
 	srcdir = dirname(dir);
-	addMacro(NULL, "_sourcedir", NULL, srcdir, RMIL_TARBALL);
+	rpmPushMacro(NULL, "_sourcedir", NULL, srcdir, RMIL_TARBALL);
 	free(dir);
     } else {
 	specFile = xstrdup(arg);
@@ -596,7 +595,12 @@ int main(int argc, char *argv[])
 
     const char *pkg = NULL;
     int ec = 0;
-    poptContext optCon = rpmcliInit(argc, argv, optionsTable);
+
+    poptContext optCon = NULL;
+
+    xsetprogname(argv[0]); /* Portability call -- see system.h */
+
+    optCon = rpmcliInit(argc, argv, optionsTable);
 
     /* Args required only when building, let lone --eval etc through */
     if (ba->buildAmount && poptPeekArg(optCon) == NULL) {
@@ -615,14 +619,19 @@ int main(int argc, char *argv[])
 	argerror(_("arguments to --root (-r) must begin with a /"));
     }
 
-    /* rpmbuild is rather chatty by default */
-    rpmSetVerbosity(quiet ? RPMLOG_WARNING : RPMLOG_INFO);
+    /* rpmbuild runs in verbose mode by default */
+    if (rpmlogSetMask(0) < RPMLOG_MASK(RPMLOG_INFO))
+	rpmSetVerbosity(RPMLOG_INFO);
+
+    if (quiet)
+	rpmSetVerbosity(RPMLOG_WARNING);
 
     if (rpmcliPipeOutput && initPipe())
 	exit(EXIT_FAILURE);
 	
     ts = rpmtsCreate();
     (void) rpmtsSetRootDir(ts, rpmcliRootDir);
+    rpmtsSetFlags(ts, rpmtsFlags(ts) | RPMTRANS_FLAG_NOPLUGINS);
 
     switch (buildChar) {
     case 'a':
