@@ -21,14 +21,10 @@
 
 #include "lib/rpmlead.h"
 #include "lib/signature.h"
-#include "lib/rpmsignfiles.h"
+#include "sign/rpmsignfiles.h"
 
 #include "debug.h"
 
-#if !defined(__GLIBC__) && !defined(__APPLE__)
-char ** environ = NULL;
-#endif
-
 typedef struct sigTarget_s {
     FD_t fd;
     const char *fileName;
@@ -121,7 +117,7 @@ static int manageFile(FD_t *fdp, const char *fn, int flags)
 
     /* open a file and set *fdp */
     if (*fdp == NULL && fn != NULL) {
-	switch(flags & O_ACCMODE) {
+	switch (flags & O_ACCMODE) {
 	    case O_WRONLY:
 		fmode = "w.ufdio";
 		break;
@@ -192,14 +188,13 @@ exit:
  * Validate generated signature and insert to header if it looks sane.
  * NSS doesn't support everything GPG does. Basic tests to see if the 
  * generated signature is something we can use.
- * Return 0 on success, 1 on failure.
+ * Return generated signature tag data on success, NULL on failure.
  */
-static int putSignature(Header sigh, int ishdr, uint8_t *pkt, size_t pktlen)
+static rpmtd makeSigTag(Header sigh, int ishdr, uint8_t *pkt, size_t pktlen)
 {
     pgpDigParams sigp = NULL;
     rpmTagVal sigtag;
-    struct rpmtd_s sigtd;
-    int rc = 1; /* assume failure */
+    rpmtd sigtd = NULL;
     unsigned int hash_algo;
     unsigned int pubkey_algo;
 
@@ -229,19 +224,17 @@ static int putSignature(Header sigh, int ishdr, uint8_t *pkt, size_t pktlen)
 	break;
     }
 
-    /* Looks sane, insert into header */
-    rpmtdReset(&sigtd);
-    sigtd.count = pktlen;
-    sigtd.data = pkt;
-    sigtd.type = RPM_BIN_TYPE;
-    sigtd.tag = sigtag;
-
-    /* Argh, reversed return codes */
-    rc = (headerPut(sigh, &sigtd, HEADERPUT_DEFAULT) == 0);
+    /* Looks sane, create the tag data */
+    sigtd = rpmtdNew();
+    sigtd->count = pktlen;
+    sigtd->data = memcpy(xmalloc(pktlen), pkt, pktlen);;
+    sigtd->type = RPM_BIN_TYPE;
+    sigtd->tag = sigtag;
+    sigtd->flags |= RPMTD_ALLOCED;
 
 exit:
     pgpDigParamsFree(sigp);
-    return rc;
+    return sigtd;
 }
 
 static int runGPG(sigTarget sigt, const char *sigfile)
@@ -257,8 +250,8 @@ static int runGPG(sigTarget sigt, const char *sigfile)
 
     namedPipeName = mkTempFifo();
 
-    addMacro(NULL, "__plaintext_filename", NULL, namedPipeName, -1);
-    addMacro(NULL, "__signature_filename", NULL, sigfile, -1);
+    rpmPushMacro(NULL, "__plaintext_filename", NULL, namedPipeName, -1);
+    rpmPushMacro(NULL, "__signature_filename", NULL, sigfile, -1);
 
     if (!(pid = fork())) {
 	char *const *av;
@@ -279,8 +272,8 @@ static int runGPG(sigTarget sigt, const char *sigfile)
 	_exit(EXIT_FAILURE);
     }
 
-    delMacro(NULL, "__plaintext_filename");
-    delMacro(NULL, "__signature_filename");
+    rpmPopMacro(NULL, "__plaintext_filename");
+    rpmPopMacro(NULL, "__signature_filename");
 
     fnamedPipe = Fopen(namedPipeName, "w");
     if (!fnamedPipe) {
@@ -343,15 +336,15 @@ exit:
  * @param ishdr		header-only signature?
  * @param sigt		signature target
  * @param passPhrase	private key pass phrase
- * @return		0 on success, 1 on failure
+ * @return		generated sigtag on success, 0 on failure
  */
-static int makeGPGSignature(Header sigh, int ishdr, sigTarget sigt)
+static rpmtd makeGPGSignature(Header sigh, int ishdr, sigTarget sigt)
 {
     char * sigfile = rstrscat(NULL, sigt->fileName, ".sig", NULL);
     struct stat st;
     uint8_t * pkt = NULL;
     size_t pktlen = 0;
-    int rc = 1; /* assume failure */
+    rpmtd sigtd = NULL;
 
     if (runGPG(sigt, sigfile))
 	goto exit;
@@ -368,7 +361,7 @@ static int makeGPGSignature(Header sigh, int ishdr, sigTarget sigt)
 
     {	FD_t fd;
 
-	rc = 0;
+	int rc = 0;
 	fd = Fopen(sigfile, "r.ufdio");
 	if (fd != NULL && !Ferror(fd)) {
 	    rc = Fread(pkt, sizeof(*pkt), pktlen, fd);
@@ -383,46 +376,13 @@ static int makeGPGSignature(Header sigh, int ishdr, sigTarget sigt)
     rpmlog(RPMLOG_DEBUG, "Got %zd bytes of GPG sig\n", pktlen);
 
     /* Parse the signature, change signature tag as appropriate. */
-    rc = putSignature(sigh, ishdr, pkt, pktlen);
+    sigtd = makeSigTag(sigh, ishdr, pkt, pktlen);
 exit:
     (void) unlink(sigfile);
     free(sigfile);
     free(pkt);
 
-    return rc;
-}
-
-static int rpmGenSignature(Header sigh, sigTarget sigt1, sigTarget sigt2)
-{
-    int ret;
-
-    ret = makeGPGSignature(sigh, 0, sigt1);
-    if (ret)
-	goto exit;
-
-    ret = makeGPGSignature(sigh, 1, sigt2);
-    if (ret)
-	goto exit;
-exit:
-    return ret;
-}
-
-/**
- * Retrieve signature from header tag
- * @param sigh		signature header
- * @param sigtag	signature tag
- * @return		parsed pgp dig or NULL
- */
-static pgpDigParams getSig(Header sigh, rpmTagVal sigtag)
-{
-    struct rpmtd_s pkt;
-    pgpDigParams sig = NULL;
-
-    if (headerGet(sigh, sigtag, &pkt, HEADERGET_DEFAULT) && pkt.data != NULL) {
-	pgpPrtParams(pkt.data, pkt.count, PGPTAG_SIGNATURE, &sig);
-	rpmtdFreeData(&pkt);
-    }
-    return sig;
+    return sigtd;
 }
 
 static void deleteSigs(Header sigh)
@@ -434,64 +394,73 @@ static void deleteSigs(Header sigh)
     headerDel(sigh, RPMSIGTAG_PGP5);
 }
 
-static int sameSignature(rpmTagVal sigtag, Header h1, Header h2)
+static int haveSignature(rpmtd sigtd, Header h)
 {
-    pgpDigParams sig1 = getSig(h1, sigtag);
-    pgpDigParams sig2 = getSig(h2, sigtag);;
+    pgpDigParams sig1 = NULL;
+    pgpDigParams sig2 = NULL;
+    struct rpmtd_s oldtd;
+    int rc = 0; /* assume no */
 
-    int rc = pgpDigParamsCmp(sig1, sig2);
+    if (!headerGet(h, rpmtdTag(sigtd), &oldtd, HEADERGET_DEFAULT))
+	return rc;
 
+    pgpPrtParams(sigtd->data, sigtd->count, PGPTAG_SIGNATURE, &sig1);
+    while (rpmtdNext(&oldtd) >= 0 && rc == 0) {
+	pgpPrtParams(oldtd.data, oldtd.count, PGPTAG_SIGNATURE, &sig2);
+	if (pgpDigParamsCmp(sig1, sig2) == 0)
+	    rc = 1;
+	pgpDigParamsFree(sig2);
+    }
     pgpDigParamsFree(sig1);
-    pgpDigParamsFree(sig2);
-    return (rc == 0);
+    rpmtdFreeData(&oldtd);
+
+    return rc;
 }
 
-static int replaceSignature(Header sigh, sigTarget sigt1, sigTarget sigt2)
+static int replaceSignature(Header sigh, sigTarget sigt_v3, sigTarget sigt_v4)
 {
-    /* Grab a copy of the header so we can compare the result */
-    Header oldsigh = headerCopy(sigh);
     int rc = -1;
+    rpmtd sigtd = NULL;
     
+    /* Make the cheaper v4 signature first */
+    if ((sigtd = makeGPGSignature(sigh, 1, sigt_v4)) == NULL)
+	goto exit;
+
+    /* See if we already have a signature by the same key and parameters */
+    if (haveSignature(sigtd, sigh)) {
+	rc = 1;
+	goto exit;
+    }
     /* Nuke all signature tags */
     deleteSigs(sigh);
 
-    /*
-     * rpmGenSignature() internals parse the actual signing result and 
-     * adds appropriate tags for DSA/RSA.
-     */
-    if (rpmGenSignature(sigh, sigt1, sigt2) == 0) {
-	/* Lets see what we got and whether its the same signature as before */
-	rpmTagVal sigtag = headerIsEntry(sigh, RPMSIGTAG_DSA) ?
-					RPMSIGTAG_DSA : RPMSIGTAG_RSA;
+    if (headerPut(sigh, sigtd, HEADERPUT_DEFAULT) == 0)
+	goto exit;
+    rpmtdFree(sigtd);
 
-	rc = sameSignature(sigtag, sigh, oldsigh);
+    /* Assume the same signature test holds for v3 signature too */
+    if ((sigtd = makeGPGSignature(sigh, 0, sigt_v3)) == NULL)
+	goto exit;
 
-    }
+    if (headerPut(sigh, sigtd, HEADERPUT_DEFAULT) == 0)
+	goto exit;
 
-    headerFree(oldsigh);
+    rc = 0;
+exit:
+    rpmtdFree(sigtd);
     return rc;
 }
 
 static void unloadImmutableRegion(Header *hdrp, rpmTagVal tag)
 {
-    struct rpmtd_s copytd, td;
+    struct rpmtd_s td;
     rpmtd utd = &td;
     Header nh;
     Header oh;
-    HeaderIterator hi;
 
     if (headerGet(*hdrp, tag, utd, HEADERGET_DEFAULT)) {
-	nh = headerNew();
 	oh = headerCopyLoad(utd->data);
-	hi = headerInitIterator(oh);
-
-	while (headerNext(hi, &copytd)) {
-	    if (copytd.data)
-		headerPut(nh, &copytd, HEADERPUT_DEFAULT);
-	    rpmtdFreeData(&copytd);
-	}
-
-	headerFreeIterator(hi);
+	nh = headerCopy(oh);
 	headerFree(oh);
 	rpmtdFreeData(utd);
 	headerFree(*hdrp);
@@ -500,9 +469,10 @@ static void unloadImmutableRegion(Header *hdrp, rpmTagVal tag)
     }
 }
 
+#ifdef WITH_IMAEVM
 static rpmRC replaceSigDigests(FD_t fd, const char *rpm, Header *sigp,
 			       off_t sigStart, off_t sigTargetSize,
-			       char *SHA1, uint8_t *MD5)
+			       char *SHA256, char *SHA1, uint8_t *MD5)
 {
     off_t archiveSize;
     rpmRC rc = RPMRC_OK;
@@ -521,10 +491,10 @@ static rpmRC replaceSigDigests(FD_t fd, const char *rpm, Header *sigp,
     }
 
     /* Set reserved space to 0 */
-    addMacro(NULL, "__gpg_reserved_space", NULL, 0, RMIL_GLOBAL);
+    rpmPushMacro(NULL, "__gpg_reserved_space", NULL, 0, RMIL_GLOBAL);
 
     /* Replace old digests in sigh */
-    rc = rpmGenerateSignature(SHA1, MD5, sigTargetSize, archiveSize, fd);
+    rc = rpmGenerateSignature(SHA256, SHA1, MD5, sigTargetSize, archiveSize, fd);
     if (rc != RPMRC_OK) {
 	rpmlog(RPMLOG_ERR, _("generateSignature failed\n"));
 	goto exit;
@@ -538,7 +508,7 @@ static rpmRC replaceSigDigests(FD_t fd, const char *rpm, Header *sigp,
     }
 
     headerFree(*sigp);
-    rc = rpmReadSignature(fd, sigp, RPMSIGTYPE_HEADERSIG, NULL);
+    rc = rpmReadSignature(fd, sigp, NULL);
     if (rc != RPMRC_OK) {
 	rpmlog(RPMLOG_ERR, _("rpmReadSignature failed\n"));
 	goto exit;
@@ -547,36 +517,34 @@ static rpmRC replaceSigDigests(FD_t fd, const char *rpm, Header *sigp,
 exit:
     return rc;
 }
+#endif
 
 static rpmRC includeFileSignatures(FD_t fd, const char *rpm,
 				   Header *sigp, Header *hdrp,
 				   off_t sigStart, off_t headerStart)
 {
+#ifdef WITH_IMAEVM
     FD_t ofd = NULL;
     char *trpm = NULL;
-    const char *key;
+    char *key;
     char *keypass;
     char *SHA1 = NULL;
+    char *SHA256 = NULL;
     uint8_t *MD5 = NULL;
-    size_t sha1len;
-    size_t md5len;
     off_t sigTargetSize;
     rpmRC rc = RPMRC_OK;
     struct rpmtd_s osigtd;
     char *o_sha1 = NULL;
-    uint8_t o_md5[16];
 
-#ifndef WITH_IMAEVM
-    rpmlog(RPMLOG_ERR, _("missing libimaevm\n"));
-    return RPMRC_FAIL;
-#endif
     unloadImmutableRegion(hdrp, RPMTAG_HEADERIMMUTABLE);
 
     key = rpmExpand("%{?_file_signing_key}", NULL);
 
-    keypass = rpmExpand("%{_file_signing_key_password}", NULL);
-    if (rstreq(keypass, ""))
+    keypass = rpmExpand("%{?_file_signing_key_password}", NULL);
+    if (rstreq(keypass, "")) {
+	free(keypass);
 	keypass = NULL;
+    }
 
     rc = rpmSignFiles(*hdrp, key, keypass);
     if (rc != RPMRC_OK) {
@@ -612,16 +580,20 @@ static rpmRC includeFileSignatures(FD_t fd, const char *rpm,
     }
 
     /* Start MD5 calculation */
-    fdInitDigest(fd, PGPHASHALGO_MD5, 0);
+    fdInitDigestID(fd, PGPHASHALGO_MD5, RPMSIGTAG_MD5, 0);
 
-    /* Write header to rpm and recalculate SHA1 */
-    fdInitDigest(fd, PGPHASHALGO_SHA1, 0);
+    /* Write header to rpm and recalculate digests */
+    fdInitDigestID(fd, PGPHASHALGO_SHA1, RPMSIGTAG_SHA1, 0);
+    fdInitDigestID(fd, PGPHASHALGO_SHA256, RPMSIGTAG_SHA256, 0);
     rc = headerWrite(fd, *hdrp, HEADER_MAGIC_YES);
     if (rc != RPMRC_OK) {
 	rpmlog(RPMLOG_ERR, _("headerWrite failed\n"));
 	goto exit;
     }
-    fdFiniDigest(fd, PGPHASHALGO_SHA1, (void **)&SHA1, &sha1len, 1);
+    fdFiniDigest(fd, RPMSIGTAG_SHA1, (void **)&SHA1, NULL, 1);
+    /* Only add SHA256 if it was there to begin with */
+    if (headerIsEntry(*sigp, RPMSIGTAG_SHA256))
+	fdFiniDigest(fd, RPMSIGTAG_SHA256, (void **)&SHA256, NULL, 1);
 
     /* Copy archive from temp file */
     if (Fseek(ofd, 0, SEEK_SET) < 0) {
@@ -638,24 +610,35 @@ static rpmRC includeFileSignatures(FD_t fd, const char *rpm,
     unlink(trpm);
 
     sigTargetSize = Ftell(fd) - headerStart;
-    fdFiniDigest(fd, PGPHASHALGO_MD5, (void **)&MD5, &md5len, 0);
+    fdFiniDigest(fd, RPMSIGTAG_MD5, (void **)&MD5, NULL, 0);
 
-    if (headerGet(*sigp, RPMSIGTAG_MD5, &osigtd, HEADERGET_DEFAULT))
-	memcpy(o_md5, osigtd.data, 16);
-
-    if (headerGet(*sigp, RPMSIGTAG_SHA1, &osigtd, HEADERGET_DEFAULT))
+    if (headerGet(*sigp, RPMSIGTAG_SHA1, &osigtd, HEADERGET_DEFAULT)) {
 	o_sha1 = xstrdup(osigtd.data);
+	rpmtdFreeData(&osigtd);
+    }
 
-    if (memcmp(MD5, o_md5, md5len) == 0 && strcmp(SHA1, o_sha1) == 0)
+    if (strcmp(SHA1, o_sha1) == 0)
 	rpmlog(RPMLOG_WARNING,
 	       _("%s already contains identical file signatures\n"),
 	       rpm);
     else
-	replaceSigDigests(fd, rpm, sigp, sigStart, sigTargetSize, SHA1, MD5);
+	replaceSigDigests(fd, rpm, sigp, sigStart, sigTargetSize, SHA256, SHA1, MD5);
 
 exit:
-if (ofd)    (void) closeFile(&ofd);
+    free(trpm);
+    free(MD5);
+    free(SHA1);
+    free(SHA256);
+    free(o_sha1);
+    free(keypass);
+    free(key);
+    if (ofd)
+	(void) closeFile(&ofd);
     return rc;
+#else
+    rpmlog(RPMLOG_ERR, _("file signing support not built in\n"));
+    return RPMRC_FAIL;
+#endif
 }
 
 /** \ingroup rpmcli
@@ -669,18 +652,17 @@ static int rpmSign(const char *rpm, int deleting, int signfiles)
 {
     FD_t fd = NULL;
     FD_t ofd = NULL;
-    rpmlead lead = NULL;
     char *trpm = NULL;
     Header sigh = NULL;
     Header h = NULL;
-    char * msg = NULL;
+    char *msg = NULL;
     int res = -1; /* assume failure */
     rpmRC rc;
     struct rpmtd_s utd;
     off_t headerStart;
     off_t sigStart;
-    struct sigTarget_s sigt1;
-    struct sigTarget_s sigt2;
+    struct sigTarget_s sigt_v3;
+    struct sigTarget_s sigt_v4;
     unsigned int origSigSize;
     int insSig = 0;
 
@@ -689,25 +671,22 @@ static int rpmSign(const char *rpm, int deleting, int signfiles)
     if (manageFile(&fd, rpm, O_RDWR))
 	goto exit;
 
-    if ((rc = rpmLeadRead(fd, &lead, NULL, &msg)) != RPMRC_OK) {
+    if ((rc = rpmLeadRead(fd, NULL, &msg)) != RPMRC_OK) {
 	rpmlog(RPMLOG_ERR, "%s: %s\n", rpm, msg);
-	free(msg);
 	goto exit;
     }
 
     sigStart = Ftell(fd);
-    rc = rpmReadSignature(fd, &sigh, RPMSIGTYPE_HEADERSIG, &msg);
+    rc = rpmReadSignature(fd, &sigh, &msg);
     if (rc != RPMRC_OK) {
 	rpmlog(RPMLOG_ERR, _("%s: rpmReadSignature failed: %s"), rpm,
 		    (msg && *msg ? msg : "\n"));
-	msg = _free(msg);
 	goto exit;
     }
 
     headerStart = Ftell(fd);
     if (rpmReadHeader(NULL, fd, &h, &msg) != RPMRC_OK) {
 	rpmlog(RPMLOG_ERR, _("%s: headerRead failed: %s\n"), rpm, msg);
-	msg = _free(msg);
 	goto exit;
     }
 
@@ -727,16 +706,16 @@ static int rpmSign(const char *rpm, int deleting, int signfiles)
 	deleteSigs(sigh);
     } else {
 	/* Signature target containing header + payload */
-	sigt1.fd = fd;
-	sigt1.start = headerStart;
-	sigt1.fileName = rpm;
-	sigt1.size = fdSize(fd) - headerStart;
+	sigt_v3.fd = fd;
+	sigt_v3.start = headerStart;
+	sigt_v3.fileName = rpm;
+	sigt_v3.size = fdSize(fd) - headerStart;
 
 	/* Signature target containing only header */
-	sigt2 = sigt1;
-	sigt2.size = headerSizeof(h, HEADER_MAGIC_YES);
+	sigt_v4 = sigt_v3;
+	sigt_v4.size = headerSizeof(h, HEADER_MAGIC_YES);
 
-	res = replaceSignature(sigh, &sigt1, &sigt2);
+	res = replaceSignature(sigh, &sigt_v3, &sigt_v4);
 	if (res != 0) {
 	    if (res == 1) {
 		rpmlog(RPMLOG_WARNING,
@@ -803,7 +782,7 @@ static int rpmSign(const char *rpm, int deleting, int signfiles)
 	}
 
 	/* Write the lead/signature of the output rpm */
-	rc = rpmLeadWrite(ofd, lead);
+	rc = rpmLeadWrite(ofd, h);
 	if (rc != RPMRC_OK) {
 	    rpmlog(RPMLOG_ERR, _("%s: writeLead failed: %s\n"), trpm,
 		Fstrerror(ofd));
@@ -840,9 +819,9 @@ exit:
     if (fd)	(void) closeFile(&fd);
     if (ofd)	(void) closeFile(&ofd);
 
-    rpmFreeSignature(sigh);
+    headerFree(sigh);
     headerFree(h);
-    rpmLeadFree(lead);
+    free(msg);
 
     /* Clean up intermediate target */
     if (trpm) {
@@ -861,11 +840,11 @@ int rpmPkgSign(const char *path, const struct rpmSignArgs * args)
 	if (args->hashalgo) {
 	    char *algo = NULL;
 	    rasprintf(&algo, "%d", args->hashalgo);
-	    addMacro(NULL, "_gpg_digest_algo", NULL, algo, RMIL_GLOBAL);
+	    rpmPushMacro(NULL, "_gpg_digest_algo", NULL, algo, RMIL_GLOBAL);
 	    free(algo);
 	}
 	if (args->keyid) {
-	    addMacro(NULL, "_gpg_name", NULL, args->keyid, RMIL_GLOBAL);
+	    rpmPushMacro(NULL, "_gpg_name", NULL, args->keyid, RMIL_GLOBAL);
 	}
     }
 
@@ -873,17 +852,17 @@ int rpmPkgSign(const char *path, const struct rpmSignArgs * args)
 
     if (args) {
 	if (args->hashalgo) {
-	    delMacro(NULL, "_gpg_digest_algo");
+	    rpmPopMacro(NULL, "_gpg_digest_algo");
 	}
 	if (args->keyid) {
-	    delMacro(NULL, "_gpg_name");
+	    rpmPopMacro(NULL, "_gpg_name");
 	}
     }
 
     return rc;
 }
 
-int rpmPkgDelSign(const char *path)
+int rpmPkgDelSign(const char *path, const struct rpmSignArgs * args)
 {
     return rpmSign(path, 1, 0);
 }
